function [state, statemap] = IPConstSim(IC, Param, iter)
% [state, statemap, u] = IPConstSim(IC, Param, iter)
%
% Inputs
% - IC -> Initial Conditions [x, z, xdot, zdot]
%       - ***COORDINATES RELATIVE TO STEP***
% - Param -> Parameters [T, w, L]
%       - T is step time
%       - w is step width
%       - L is step length
% - iter -> Number of Iterations
%
% Outputs
% - state -> State variables [x, z, xcom, zcom, time]
%       - Each variable is its own column
% - statemap -> State variables at each step [xstep, zstep, xcomstep,
%               zcomstep, tstep]
% - u -> Steps [ux uz]
%
% William Corvino
% 14-April-2023

% Pull Initial Conditions
xinit = IC(1);
zinit = IC(2);
xdotinit = IC(3);
zdotinit = IC(4);

% Pull Parameters
T = Param(1);
w = Param(2);
L = Param(3);
w0 = sqrt(9.81/0.4);

% Creates time vector
t = linspace(0, T, 101);

% Creates state storage vectors
x = ones(iter*100 + 1, 1);
z = ones(iter*100 + 1, 1);
xdot = ones(iter*100 + 1, 1);
zdot = ones(iter*100 + 1, 1);
time = ones(iter*100 + 1, 1);

% Creates map storage vectors
xstep = ones(iter + 1, 1);
zstep = ones(iter + 1, 1);
xdotstep = ones(iter + 1, 1);
zdotstep = ones(iter + 1, 1);
tstep = 0:T:iter*T;

% Sets first values for map
xstep(1) = xinit;
zstep(1) = zinit;
xdotstep(1) = xdotinit;
zdotstep(1) = zdotinit;

% Simulation
for i = 1:iter

    % Pulls step initial conditions
    x0 = xstep(i);
    z0 = zstep(i);
    xdot0 = xdotstep(i);
    zdot0 = zdotstep(i);

    % Evaluates continuous-time state
    xtemp = cosh(w0*t) * x0 + 1/w0 * sinh(w0*t) * xdot0;
    ztemp = cosh(w0*t) * z0 + 1/w0 * sinh(w0*t) * zdot0;
    xdottemp = w0 * sinh(w0*t) * x0 + cosh(w0*t) * xdot0;
    zdottemp = w0 * sinh(w0*t) * z0 + cosh(w0*t) * zdot0;

    % Fills state storage containers
    x((i-1)*100 + 1:i*100) = xtemp(1:100);
    z((i-1)*100 + 1:i*100) = ztemp(1:100);
    xdot((i-1)*100 + 1:i*100) = xdottemp(1:100);
    zdot((i-1)*100 + 1:i*100) = zdottemp(1:100);
    time((i-1)*100 + 1:i*100) = t(1:100) + (i-1)*T;

    % Applies Step Transition Map
    xtemp(end) = xtemp(end) - L;
    ztemp(end) = -ztemp(end) + w;
    xdottemp(end) = xdottemp(end);
    zdottemp(end) = -zdottemp(end);
    
    % Fills map storage containers
    xstep(i+1) = xtemp(end);
    zstep(i+1) = ztemp(end);
    xdotstep(i+1) = xdottemp(end);
    zdotstep(i+1) = zdottemp(end);

end

% Adds final value to end
x(end) = xtemp(end);
z(end) = ztemp(end);
xdot(end) = xdottemp(end);
zdot(end) = zdottemp(end);
time(end) = iter*T;

% Output
state = [x, z, xdot, zdot, time];
statemap = [xstep, zstep, xdotstep, zdotstep, tstep'];

end