function varargout = IPModelSim(details, varargin)
% IPModelSim(<details>, <stepdetails>, <controlMode>, <initialConditions>, <noiseDist>, <specData>, <otherParameters>)
% 
% Runs a simulation of the Inverted Pendulum model
% Displays data using Hof's formulation of the extrapolated center of mass
%
% details -> Specifies which details will be specified in command call
%   'S' -> Step Details (Details about the plotting of steps)
%   'C' -> Control Mode (If steps aren't specified, what controller should
%   the code use?
%   'I' -> Initial Conditions
%   'N' -> Noise (If steps aren't specified, how much noise should there
%   be?)
%   'Z' -> Specified z Step Locations
%   'X' -> Specified x Step Locations
%   '2' -> Quadratic
%   'T' -> Specified Step Times
%   'A' -> Specified Step Angles (Mutually exclusive with 'X' and 'Z')
%   'P' -> Other Parameters
%
% Inputs MUST appear in this order, skipping over non-specified inputs.
%
% stepdetails -> [cycles, firststep, [plotSteps]]
%
% - cycles = number of steps (left and right)
%       Overwritten in almost all cases
%       If cycles are not provided:
%           - Default: 10
%           - If data is provided, cycles will the number of steps provided
%
% - firstStep = 0 (Left Foot Down) or 1 (Right Foot Down)
%       Left by default
%
% - plotSteps = [skipsteps, endStep, <mosstep>]
%   - Controls what steps are plotted
%   - Note: One 'Step' = One Foot Transition (or one step placement)
%
%   skipsteps -> Number of skipped steps
%       - Default: 0
%   endStep -> Last step to be plotted
%       - Default: Last step (same as 'cycles')
%   mosstep -> Chooses which step side will be shown on certain graphs
%       - Default: 0 (Left)
% 
% controlMode -> all the control options
%   'C' -> Constant Offset based on constant step width and length
%   '+' -> Constant Offset plus proportional error from target COM
%   'C' by default
% 
% initialConditions = input initial conditions
%   - Default: [0 0 0 0]
%   [x0, z0] -> Initial COM Position
%       - Equivalent to [x0 z0 0 0]
%   [x0, z0, xcom0, zcom0] -> All initial conditions
% 
% noiseDist = Noise Standard Deviations
%   [sigmat] -> Time standard deviation
%   [sigmax, sigmaz] -> x and z foot placement standard deviations
%   [sigmax, sigmaz, sigmat] -> All standard deviations
%
% specData = Specified Steps, Times, and Angle Data
%   - MUST be in columns as [<uxspec>, <uzspec>, <tspec>, <aspec>]
%   uxspec -> Sagittal Center of Pressure (CoP) locations for each step
%   uzspec -> Lateral Center of Pressure locations for each step
%   tspec -> Step Time
%   aspec -> Step Angles/Heading
%       - Angles must be specified as counterclockwise rotation
%       - Angle specification is overwritten by ux and uz specification
%
% controlParameters = [f, T, L, w, s, k1, goalz]
%   goalz can be specified as a row vector with goalz values for each step
%   Defaults
%       - f = 100 [Hz] -> Sampling Frequency
%       - T = 0.5 [s] -> Constant step time (overwritten by tspec)
%       - L = 0.4 [m] -> Leg/Pendulum Length
%       - w = 0.1 [m] -> Constant step width (overwritten by uzspec)
%       - s = 0.75 [m] -> Constant step length (overwritten by uxspec)
%       - k1 = 0.1 -> Gain for 'C+' control
%       - goalz = 0.05 [m] -> Target CoM position for 'C+' control
%
%
%   William Corvino
%   16 - Sep - 2022
%   v1.2

%% References
% Hof '08: "The "extrapolated center of mass’ concept suggests a simple control of balance in walking"
% Hof '05: "The condition for dynamic stability"
% Kazanski et. al '21: "Rethinking Margin of Stability: Incorporating Step-To-Step Regulation to Resolve the Paradox"
% Dingwell & Cusumano '19: "Humans use multi-objective control to regulate lateral foot placement when walking"


%% To-Do
% Just Z-direction graph

%% Defaults
    % Other Parameters
    f = 100;
    T = 0.5;
    w = 0.1;
    s = 0.75;
    k1 = 0.1;
    goalz = 0.05;
    L = 0.4;
    g = 9.81;
    w0 = sqrt(g/L);

    % Step Details
    cycles = 10;
    firststep = 0;

    % Control Mode
    controlMode = 'C';

    % Initial Conditions
    x0 = 0;
    z0 = 0;
    xcom0 = 0;
    zcom0 = 0;

    % Noise
    uxdist = 0;
    uzdist = 0;
    tdist = 0;


%% Input Arguments

% Defaults (All not specified)
output = 0; % Output Step Data
sdetail = 0; % Step Details
controldetails = 0; % Control Mode
initialspec = 0; % Initial Conditions
noise = 0; % Noise Amplitude
xspec = 0; % Specified X Steps
zspec = 0; % Specified Z Steps
quad = 0;
timespec = 0; % Specified Step Times
aspec = 0; % Specified Step Angles
paraspec = 0; % Other Parameters

try % Checks for symbol for each specification
    detailString = char(details); % Creates character array from the string
    if(contains(detailString, 'O'))
        output = 1;
    end
    if(contains(detailString, 'S'))
        sdetail = 1;
    end
    if(contains(detailString, 'C'))
        controldetails = 1;
    end
    if(contains(detailString, 'I'))
        initialspec = 1;
    end
    if(contains(detailString, 'N'))
        noise = 1;
    end
    if(contains(detailString, 'Z'))
        zspec = 1;
    end
    if(contains(detailString, 'X'))
        xspec = 1;
    end
    if(contains(detailString, '2'))
        quad = 1;
    end
    if(contains(detailString, 'A'))
        aspec = 1;
    end
    if(contains(detailString, 'T'))
        timespec = 1;
    end
    if(contains(detailString, 'P'))
        paraspec = 1;
    end
catch
    % If nothing is provided, set all to defaults
    output = 0;
    sdetail = 0;
    controldetails = 0;
    initialspec = 0;
    xspec = 0;
    zspec = 0;
    timespec = 0;
    aspec = 0;
    paraspec = 0;

end

% Extracts inputs from the function input
% Converts to format from earlier versions
i = 1;
    if(sdetail) % Step Details
        stepdetails = varargin{i};
        if(length(stepdetails) == 1)
            cycles = stepdetails(1); % # of Cycles
            firststep = 0; % First Step
        end
        if(length(stepdetails) >= 2)
            cycles = stepdetails(1);
            firststep = stepdetails(2);
        end
        if(length(stepdetails) >= 3)
            plotSteps = stepdetails(3:end); % Other Plotting Details
        end
        i = i + 1;
    end
    if(controldetails)
        controlMode = varargin{i}; % Control Mode
        i = i + 1;
    end
    if(initialspec)
        initialConditions = varargin{i}; % Initial Conditions
        i = i + 1;
    end
    if(noise)
        noiseAmp = varargin{i}; % Noise Amplitude
        i = i + 1;
    end
    if(zspec || xspec || timespec || aspec)
        specData = varargin{i}; % Specified Data
        i = i + 1;
    end
    if(paraspec)
        parameters = varargin{i}; % Other Parameters
    end

%%
% Parses 'Other Parameters' for random model parameters
try
    if(length(parameters) >= 1) % Sampling Frequency
        f = parameters(1);
    end
    if(length(parameters) >= 2) % Step Time
        T = parameters(2);
    end
    if(length(parameters) >= 3) % Leg Length
        L = parameters(3);
        w0 = sqrt(g/L);
    end
    if(length(parameters) >= 4) % Controlled Step Width
        w = parameters(4);     
    end
    if(length(parameters) >= 5) % Controlled Step Length
        s = parameters(5);   
    end
    if(length(parameters) >= 6) % Control Parameter
        k1 = parameters(6);  
    end
    if(length(parameters) >= 7) % Target Lateral Position
        goalz = parameters(7:end);
    end
catch
    % If there is an error, reset to defaults
    f = 100;
    T = 0.5;
    w = 0.1;
    s = 0.75;
    k1 = 0.1;
    goalz = 0.05;
    L = 0.4;
    w0 = sqrt(g/L);
end

% Parses Control Mode
try
    controlString = char(controlMode);
    
    % Constant Offset
    c0 = contains(controlString, 'C') || isempty(controlString);

    % Constant Plus Proportional
    c1 = contains(controlString, '+');

    % Default Checking
    if(~c1)
        c0 = true;
    end

catch
    % Reset to Defaults
    c0 = true;
    c1 = false;

end

% Parse Initial Conditions
try
    if(length(initialConditions) == 2)
        x0 = initialConditions(1);
        z0 = initialConditions(2);
        xcom0 = x0;
        zcom0 = z0;
    elseif(length(initialConditions) == 4)
        x0 = initialConditions(1);
        z0 = initialConditions(2);
        xcom0 = initialConditions(3);
        zcom0 = initialConditions(4);
    elseif(~isempty(initialConditions))
        error('Invalid initial conditions. Initial conditions must be of the format [x0, z0] or [x0, z0, xcom0, zcom0]')
    end
catch
    % Reset to Defaults
    x0 = 0;
    z0 = 0;
    xcom0 = 0;
    zcom0 = 0;

end

% Parse Noise Amplitudes
try
    % If only one noise term: Time Noise
    if(length(noiseAmp) == 1)
        tdist = noiseAmp(1);
        uxdist = 0;
        uzdist = 0;
    end

    % If 2 or more noise terms: Step Position Noise
    if(length(noiseAmp) >= 2 && length(noiseAmp) < 4)
        uxdist = noiseAmp(1);
        uzdist = noiseAmp(2);
        tdist = 0;
    end

    % If 3 noise terms: Both types of Noise
    if(length(noiseAmp) == 3)
        tdist = noiseAmp(3);
    end
catch
    % Reset to Defaults
    uxdist = 0;
    uzdist = 0;
    tdist = 0;
end

% Parses Specified Data
try
    count = 1;

    if(xspec) % X Step Positions
        uxspec = specData(:, count:(count + 2 * quad));
        count = count + 1 + 2 * quad;
    end
    if(zspec) % Z Step Positions
        uzspec = specData(:, count:(count + 2 * quad));
        count = count + 1 + 2 * quad;
    end % Step Time
    if(timespec)
        tspec = specData(:, count);
        count = count + 1;
    end % Step Angle
    if(aspec)
        angle = specData(:, count);
        count = count + 1;
    end
    
    % If any data is specified, set cycles to the appropriate size
    if(count ~= 1)
        cycles = size(specData, 1) - 1;
    end
 
    % If time is not specified, create the default time vector
    if(~timespec)
        tspec = ones(1, cycles) * T;
    end
catch
    % Reset to Defaults
    tspec = ones(1, cycles) * T;
end

% Parses plotSteps [skipsteps, endStep, <mosstep>]
try
    skipsteps = plotSteps(1); % Number of skipped steps
    endStep = plotSteps(2); % Last plotted step
    mosstep = 0; % Step for MoS plot (Left by default)
    if(length(plotSteps) == 3)
        mosstep = plotSteps(3); % If provided, set MoS Step
    end

    % Ensure endStep is not too large
    if(endStep > cycles)
        endStep = cycles;
    end
catch
    skipsteps = 0;
    endStep = cycles;
    mosstep = 0;
end

%% Control
% Constant Offset
if(~xspec)
    uxspace = s/(exp(w0*T)-1); % Sagittal; Also Constant Plus Proportional Controller
end

if(c0 && ~c1 && ~zspec)
    uzspace = w/(exp(w0*T)+1); % Lateral
end

% Constant Plus Proportional
if(c1 && ~zspec)
    uzspace = w/((exp(w0*T)+1)/(1 - k1/2*(exp(w0*T) - 1))); % Lateral
end

% Center of Pressure Space Vector
try
    uspace = [uxspace; uzspace];
catch
    uspace = [0; 0];
    if(~xspec)
        uspace = uspace + [uxspace; 0];
    end
    if(~zspec)
        uspace = uspace + [0; uzspace];
    end
end


%% Simulation

% Data Storage Containers

totalsteps = round(sum(tspec(1:length(tspec)-1)) * f);

x = zeros(1, totalsteps + 1);
x(1) = x0;
z = zeros(1, totalsteps + 1);
z(1) = z0;
xcom = zeros(1, totalsteps + 1);
xcom(1) = xcom0;
zcom = zeros(1, totalsteps + 1);
zcom(1) = zcom0;

time = zeros(1, totalsteps);
xdiff = zeros(1, totalsteps);
ux = zeros(1+2*quad, cycles);
uz = zeros(1+2*quad, cycles);
tstep = zeros(1, cycles);
tstep(1) = 0;

endx = zeros(1, cycles + 1);
endx(1) = x0;
endz = zeros(1, cycles + 1);
endz(1) = z0;
endxcom = zeros(1, cycles + 1);
endxcom(1) = xcom0;
endzcom = zeros(1, cycles + 1);
endzcom(1) = xcom0;

% Figure Setup
HofModel = figure(1);
HofModel.Name = 'Hof Simulation';
HofModel.Position = [430 250 1200 650];
clf(HofModel);
HofLayout = tiledlayout(3, 7);
HofLayout.Padding = 'compact';

% Absolute Position Tile
nexttile([3 3])

tcycles = cycles - firststep;

curlength = 0;
lengths = ones(1, cycles);

for i = 1:tcycles

    % Time
    t = 0:1/f:(tspec(i) + tdist * randn);

    if(t(end) ~= tspec(i))
        t = [t, tspec(i)];
    end

    % Flips default step side if first step is a right step (1)
    j = i + firststep;
    
    % Specified Angles
    if(aspec)
        % Uses a rotation matrix to rotate the vector for the next step
        uspacetemp = [cosd(angle(i)) sind(angle(i)); -sind(angle(i)) cosd(angle(i))] * [-1 0; 0 (-1)^j] * uspace;
    else
        % Default
        uspacetemp = [-1 0; 0 (-1)^j] * uspace;
    end

    % Sets spacing equal to determined values
    uxspace = uspacetemp(1);
    uzspace = uspacetemp(2);
    
    % Placement of Center of Pressure

    % Constant Offset Sagittal
    uxtemp = xcom(curlength + 1) + uxspace + uxdist * randn;
        
    % Constant Offset Lateral
    if(c0 && ~c1)
        uztemp = zcom(curlength + 1) + uzspace + uzdist * randn;
    end
    % Constant Plus Proportional Lateral
    if(c1)
        uztemp = zcom(curlength + 1) + uzspace + k1 * (zcom(curlength + 1) - goalz) + uzdist * randn;
    end

    % If the steps are specified, set them (with noise)
    if(zspec)
        uztemp = uzspec(j, :) + uzdist * randn;
    end
    if(xspec)
        uxtemp = uxspec(j, :) + uxdist * randn;
    end

    % Writes to the step position arrays
    ux(:, i) = uxtemp;
    uz(:, i) = uztemp;

    if(i ~= cycles && ~timespec && ~aspec && (zspec || xspec))

        wn = abs(uzspec(j + 1) - uzspec(j));
        bz = uz(1+2*quad, i) - zcom(curlength + 1);

        if(i < 2)
            wn1 = wn;
            wn2 = wn;
        end
        if(i < 3)
            wn2 = wn;
        end

        %{
        if(abs(uzspec(j+1) - uzspec(j)) - abs(uz(i) - zcom((i-1)*100 + 1)) <= 0)
            ti = tmin;
        else
            ti = 1/w0 * log(abs(uzspec(j + 1) - uzspec(j))/abs(uz(i) - zcom((i - 1)*100 + 1)) - 1);
        end

        if ti < tmin
            ti = tmin;
        end
        %}

        %tz = 0.12*(0.6 * wn  +  0.3 * wn1 + 0.1 * wn2);
        tz = 0.12*wn;
        %tz = abs(bz);
        ti = 1/w0 * log(abs(wn - tz)/abs(bz));
        % Project Forward a Step
        %{
        zcomfut = uztemp + (zcom((i - 1) * 100 + 1) - uztemp) * exp(w0 * ti);
        tifut = 1/w0 * log(abs(uzspec(j + 2) - uzspec(j + 1))/abs(uz(i + 1) - zcomfut) - 1);

        if(abs(uzspec(j+1) - uzspec(j)) - abs(uz(i) - zcom((i-1)*100 + 1)) <= 0 || tifut < tmin)
            Tz = abs(uzspec(j+2) - uzspec(j + 1))/(1 + exp(w0*tmin));
            ti = 1/w0 * log((wn - Tz)/bz);
        end
        [i ti]
        %}
        t = linspace(0, ti);
        %timearray(i) = ti;
        if(i > 1)
            wn2 = wn1;
        end
        wn1 = wn;
        
    end
    
    if(quad)

        % Solution for Center of Mass

        c2x = uxtemp(1);
        c1x = uxtemp(2);
        c0x = uxtemp(3);
        c2z = uztemp(1);
        c1z = uztemp(2);
        c0z = uztemp(3);

        xinit = x(curlength + 1);
        zinit = z(curlength + 1);
        xcominit = xcom(curlength + 1);
        zcominit = zcom(curlength + 1);

        asdf = c0z - zinit;

        vxinit = (xcominit-xinit)*w0;
        vzinit = (zcominit-zinit)*w0;

        xtemp = cosh(w0*t)*xinit + sinh(w0*t)*vxinit/w0 - c1x*(sinh(w0*t)-w0*t)/w0 - c2x*(2*cosh(w0*t) - w0^2 * t.^2 - 2)/w0^2 - c0x*(cosh(w0*t)-1);
        ztemp = cosh(w0*t)*zinit + sinh(w0*t)*vzinit/w0 - c1z*(sinh(w0*t)-w0*t)/w0 - c2z*(2*cosh(w0*t) - w0^2 * t.^2 - 2)/w0^2 - c0z*(cosh(w0*t)-1);

        vxtemp = w0*sinh(w0*t)*xinit + cosh(w0*t)*vxinit - c1x*(cosh(w0*t) - 1) - c2x*(2*sinh(w0*t) - 2*w0*t)/w0 - c0x*w0*(sinh(w0*t));
        vztemp = w0*sinh(w0*t)*zinit + cosh(w0*t)*vzinit - c1z*(cosh(w0*t) - 1) - c2z*(2*sinh(w0*t) - 2*w0*t)/w0 - c0z*w0*(sinh(w0*t));

        xcomtemp = xtemp + vxtemp/w0;
        zcomtemp = ztemp + vztemp/w0;

    else

        % Solution for Center of Mass
        xtemp = uxtemp + 1/2 * (xcom(curlength + 1) - uxtemp) * exp(w0 * t) + (x(curlength + 1) - 1/2 * uxtemp - 1/2 * xcom(curlength + 1)) * exp(-w0 * t);
        ztemp = uztemp + 1/2 * (zcom(curlength + 1) - uztemp) * exp(w0 * t) + (z(curlength + 1) - 1/2 * uztemp - 1/2 * zcom(curlength + 1)) * exp(-w0 * t);

        % Solution for Extrapolated Center of Mass
        xcomtemp = uxtemp + (xcom(curlength + 1) - uxtemp) * exp(w0 * t);
        zcomtemp = uztemp + (zcom(curlength + 1) - uztemp) * exp(w0 * t);

    end

    % Fills in data vectors
    x(curlength + 2:curlength + length(t)) = xtemp(2:length(t));
    z(curlength + 2:curlength + length(t)) = ztemp(2:length(t));
    xcom(curlength + 2:curlength + length(t)) = xcomtemp(2:length(t));
    zcom(curlength + 2:curlength + length(t)) = zcomtemp(2:length(t));

    endx(i + 1) = xtemp(end);
    endz(i + 1) = ztemp(end);
    endxcom(i + 1) = xcomtemp(end);
    endzcom(i + 1) = zcomtemp(end);

    xdiff(curlength + 1:curlength + length(t)-1) = (xtemp(1:length(t)-1) - ones(1, length(xtemp(1:length(t)-1)))*uxtemp(1+2*quad));

    try
        time(curlength + 1:curlength + length(t)-1) = t(1:length(t)-1) + sum(tspec(1:i-1));
        tstep(i) = sum(tspec(1:i-1));
    catch
        time(curlength + 1:curlength + length(t)-1) = t(1:length(t)-1);
        tstep(1) = 0;
    end

    curlength = curlength + length(t)-1;
    lengths(i) = curlength;

end

lengths = [0 lengths];

x(curlength+1:end) = [];
xcom(curlength+1:end) = [];
z(curlength+1:end) = [];
zcom(curlength+1:end) = [];

% x = [x, endx(end)];
% z = [x, endx(end)];
% xcom = [x, endx(end)];
% zcom = [x, endx(end)];

if(output)
    if nargout >= 1
        varargout{1} = uz;
    end
    if nargout >= 2
        varargout{2} = ux;
    end
end

% Fix THIS!!!
if(~output)
    if nargout >= 1
        varargout{1} = x;
    end
    if nargout >= 2
        varargout{2} = z;
    end
    if nargout >= 3
        varargout{3} = xcom;
    end
    if nargout >= 4
        varargout{4} = zcom;
    end
    if nargout >= 5
        varargout{5} = time;
    end
    if nargout >= 6
        varargout{6} = ux;
    end
    if nargout >= 7
        varargout{7} = uz;
    end
    if nargout >= 8
        varargout{8} = tstep;
    end
end

% Plots Center of Pressure Points
hold on
u = plot(uz(1+2*quad, skipsteps + 1:endStep), ux(1+2*quad, skipsteps + 1:endStep), 'r*');
hold off

% Plots Center of Mass Points (at Step Transition)
hold on
endCoM = plot(endz(skipsteps + 1:endStep), endx(skipsteps + 1:endStep), 'bo');
hold off

% Plots Extrapolated Center of Mass Lines
hold on
XCoM = plot(zcom(lengths(skipsteps+1) + 1:lengths(endStep)+1), xcom(lengths(skipsteps+1) + 1:lengths(endStep)+1), 'r');
hold off

% Plots Center of Mass Curves
hold on
CoM = plot(z(lengths(skipsteps+1) + 1:lengths(endStep)+1), x(lengths(skipsteps+1) + 1:lengths(endStep)+1), 'b');
hold off

% Fills out the rest of the plot properties
title('Stepping Motion')
xlabel('Lateral Position (m)')
ylabel('Sagittal Position (m)')
legend([XCoM, CoM, u, endCoM], {'XCoM', 'CoM', 'CoP', 'CoM at Transition'})

%% State Spaces
figure(1)
% Creates true velocity vectors
xvel = (xcom - x)*w0;
zvel = (zcom - z)*w0;
%endxvel = (endxcom - endx);
endzvel = (endzcom - endz);

endt = round(sum(tspec(1:endStep))*f);
startt = round(sum(tspec(1:skipsteps))*f);

% x state space
nexttile(4)
plot(xdiff(startt+1:endt), xvel(startt+1:endt));
xlabel('Sagittal Position (m)')
ylabel('Sagittal Velocity (m/s)')
title('Sagittal State Space')

% z state space
nexttile(5)
plot(z(lengths(skipsteps+1) + 1:lengths(endStep)+1), zvel(lengths(skipsteps+1) + 1:lengths(endStep)+1));
xlabel('Lateral Position (m)')
ylabel('Lateral Velocity (m/s)')
title('Lateral State Space')

%% MoS Graph (Kazanski '21 Fig. 5)
nexttile(11, [2 2])

% Sets up to collect correct side (left or right)
if(mod(skipsteps,2) && ~mosstep)
    skipsteps = skipsteps - 1;
elseif(~mod(skipsteps, 2) && mosstep)
    skipsteps = skipsteps - 1;
end

% Finds the Margin of Stability (Hof 05) at the step transition
endMoS = uz(1+2*quad, skipsteps + 1:2:endStep) - endzcom(skipsteps + 1:2:endStep);

% Finds the average MoS
avg = mean(endMoS);

% Plots the points w/ Kazanski et. al Axes
plot(endz(skipsteps + 1:2:endStep) - uz(1+2*quad, skipsteps + 1:2:endStep), endzvel(skipsteps + 1:2:endStep), 'b*');

hold on
% Plots Unstable Subspace
f = @(x) x;
fplot(f, [-0.2 0.2], 'r--');

% Plots Stable Subspace
g = @(x) -x;
fplot(g, [-0.2 0.2], 'r--');

% Plots MoS = avg(MoS)
h = @(x) -x - avg;
hplot = fplot(h, [-0.2 0.2], 'k-.');

% Sets Axis Size
axis([-0.2 0.2 -0.2 0.2])
hold off

% Sets remaining plot properties
legend(hplot, 'Average MoS')
xlabel('Relative Lateral Position (z - uz)')
ylabel('Lateral Velocity (zdot/w0)')
title('Lateral State Space')

%% Step Number vs. Lateral Position (Dingwell & Cusumano '19 Figures)
nexttile(6, [3, 2])

% Plots lateral step position vs. step number
plot(uz(1+2*quad, skipsteps + 1:endStep), skipsteps + 1:endStep, 'k--')

% Plots step transition markers
uzplot = uz(1+2*quad, skipsteps + 1:endStep);
nplot = skipsteps + 1:endStep;
nlogic = mod(nplot, 2) == 0;

hold on
plot(uzplot(nlogic == mosstep), nplot(nlogic == mosstep), 'r.', 'MarkerSize', 10)
hold off
hold on
plot(uzplot(nlogic ~= mosstep), nplot(nlogic ~= mosstep), 'b.', 'MarkerSize', 10)
hold off

% Sets ticks to be constant distance apart
yticks(skipsteps:ceil((endStep - skipsteps)/10):endStep)

% Sets remaining plot properties
xlabel('Lateral Step Position (uz)')
ylabel('Step Number')
title('Discrete Time uz')

end